/*
 *
 * X DLX Simulator
 *
 * The DLX is a 32-bit RISC machine based on the MIPS 2000 architecture.
 * The architecture includes 32 32-bit general purpose registers and a 
 * program counter.
 *
 * Floating point registers and an interrupt register are not implemented.
 *
 * The address space is 4 Gbytes, big-endian byte-addressable using 32-bit
 * addresses. All instructions must be properly aligned.
 *
 *
 * Cameron Humphries
 * Department of Computer Science
 * University of Adelaide
 * 1992
 *
 */
 
#include <stdio.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/Dialog.h>

#include "MemList.h"
#include "sim.h"

#define SHOW_BYTES 1
#define SHOW_HALFWORDS 2
#define SHOW_WORDS 3

#define SHOW_HEX 1
#define SHOW_SIGNED_DEC 2
#define SHOW_UNSIGNED_DEC 3
#define SHOW_OCTAL 4
#define SHOW_ASCII 5

#define SHOW_MEM_OCTAL 1
#define SHOW_MEM_DECIMAL 2
#define SHOW_MEM_HEX 3


#define NO_SELECTION 0
#define CODE_SELECTED 1
#define DATA_SELECTED 2

extern int num_kb;
extern XtAppContext app_context;
extern Widget form;
extern Widget code_list;
extern int selection_type;
extern int mem_loc_select;

char **data_arr;
int val_type = SHOW_SIGNED_DEC;
int data_type = SHOW_WORDS;
static int mem_type = SHOW_MEM_DECIMAL;
int current_data_selection = -1;

static void mem_octal();
static void mem_decimal();
static void mem_hex();
static void octal_val();
static void sign_dec_val();
static void unsign_dec_val();
static void hex_val();
static void ascii_val();
static void bytes_val();
static void half_word_val();
static void word_val();
static void edit_mem_location();
static void dialog_ok(),dialog_cancel();
static void data_select();

Widget data_list,data_label;
static Widget dialog_widg;

/*
 * 
 */
void Create_Data_View(Widget form)
{
  Widget w,popup_w,mem_popup;
  static Arg view_args[] = {
    {XtNfromVert, (XtArgVal)NULL},
    {XtNwidth, (XtArgVal)300},
    {XtNheight, (XtArgVal)350},
    {XtNallowVert, (XtArgVal)True},
    {XtNallowHoriz, (XtArgVal)True},
    {XtNuseBottom, (XtArgVal)True},
  };
  static Arg label_args[] = {
    {XtNfromVert, (XtArgVal)NULL},
    {XtNlabel, (XtArgVal)"DATA (address: UNS, data: DEC)"},
    {XtNborderWidth, (XtArgVal)0},
  };
  static Arg list_args[] = {
    {XtNlist, (XtArgVal)NULL},
    {XtNdefaultColumns, (XtArgVal)0},
    {XtNforceColumns, (XtArgVal)True},
    {XtNnumberStrings, (XtArgVal)0},
    {XtNverticalList, (XtArgVal)True},
    {XtNcolumnSpacing, (XtArgVal)50},
    {XtNcallback, (XtArgVal)NULL},
  };
  static Arg menu_entry_args[] = {
    {XtNlabel, (XtArgVal)NULL},
    {XtNcallback, (XtArgVal)NULL},
  };
  static XtCallbackRec mem_octal_cbk[] = {
    {mem_octal,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec mem_decimal_cbk[] = {
    {mem_decimal,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec mem_hex_cbk[] = {
    {mem_hex,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec octal_cbk[] = {
    {octal_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec sign_decimal_cbk[] = {
    {sign_dec_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec unsign_decimal_cbk[] = {
    {unsign_dec_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec hex_cbk[] = {
    {hex_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec ascii_cbk[] = {
    {ascii_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec bytes_cbk[] = {
    {bytes_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec half_word_cbk[] = {
    {half_word_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec word_cbk[] = {
    {word_val,NULL},
    {NULL,NULL},
  };
  static XtCallbackRec list_cbk[] = {
    {data_select,NULL},
    {NULL,NULL},
  };
  static Arg dialog_args[] = {
    {XtNlabel, (XtArgVal)"Edit memory location:"},
    {XtNvalue, (XtArgVal)""},
  };
  static String translations = 
    "<Btn3Down>: XawPositionSimpleMenu(data_menu) MenuPopup(data_menu)\n\
     <Btn1Down>(2): edit_mem()\n\
     <Btn1Down>: Set() Notify()";
  static XtActionsRec act_table[] = {
    {"edit_mem",edit_mem_location},
    {"DataDialogOK",dialog_ok},
  };
  static String label_trans = 
    "<Btn3Down>: XawPositionSimpleMenu(mem_data_menu) MenuPopup(mem_data_menu)";
  static String data_dialog_trans = 
    "<Key>Return:DataDialogOK()\n\
     <Key>KP_Enter:DataDialogOK()";
    
  XtAppAddActions(app_context,act_table,XtNumber(act_table));
  
  /* Create dialog widget for editing values */
  w = XtCreatePopupShell("data_edit",overrideShellWidgetClass,form,NULL,0);
  dialog_widg = XtCreateManagedWidget("data_dialog",dialogWidgetClass,w,
				      dialog_args,XtNumber(dialog_args));
  XawDialogAddButton(dialog_widg,"OK",dialog_ok,NULL);
  XawDialogAddButton(dialog_widg,"Cancel",dialog_cancel,NULL);
  XtOverrideTranslations(XtNameToWidget(dialog_widg,"value"),
			 XtParseTranslationTable(data_dialog_trans));

  /* Create popup representation menu */
  popup_w = XtCreatePopupShell("data_menu",simpleMenuWidgetClass,form,NULL,0);
  
  menu_entry_args[0].value = (XtArgVal)"OCTAL";
  menu_entry_args[1].value = (XtArgVal)octal_cbk;
  w = XtCreateManagedWidget("octal",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"SIGNED DECIMAL";
  menu_entry_args[1].value = (XtArgVal)sign_decimal_cbk;
  w = XtCreateManagedWidget("sdec",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"UNSIGNED DECIMAL";
  menu_entry_args[1].value = (XtArgVal)unsign_decimal_cbk;
  w = XtCreateManagedWidget("usdec",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"HEX";
  menu_entry_args[1].value = (XtArgVal)hex_cbk;
  w = XtCreateManagedWidget("hex",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"ASCII";
  menu_entry_args[1].value = (XtArgVal)ascii_cbk;
  w = XtCreateManagedWidget("ascii",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  w = XtCreateManagedWidget("Line_1",smeLineObjectClass,popup_w,NULL,0);

  menu_entry_args[0].value = (XtArgVal)"BYTES";
  menu_entry_args[1].value = (XtArgVal)bytes_cbk;
  w = XtCreateManagedWidget("bytes",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"HALF WORDS";
  menu_entry_args[1].value = (XtArgVal)half_word_cbk;
  w = XtCreateManagedWidget("halfwords",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"WORDS";
  menu_entry_args[1].value = (XtArgVal)word_cbk;
  w = XtCreateManagedWidget("words",smeBSBObjectClass,popup_w,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  /* Create popup memory representation menu */
  mem_popup = XtCreatePopupShell("mem_data_menu",simpleMenuWidgetClass,form,
				 NULL,0);
  
  menu_entry_args[0].value = (XtArgVal)"OCTAL";
  menu_entry_args[1].value = (XtArgVal)mem_octal_cbk;
  w = XtCreateManagedWidget("octal",smeBSBObjectClass,mem_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"DECIMAL";
  menu_entry_args[1].value = (XtArgVal)mem_decimal_cbk;
  w = XtCreateManagedWidget("sdec",smeBSBObjectClass,mem_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  menu_entry_args[0].value = (XtArgVal)"HEX";
  menu_entry_args[1].value = (XtArgVal)mem_hex_cbk;
  w = XtCreateManagedWidget("hex",smeBSBObjectClass,mem_popup,
			    menu_entry_args,XtNumber(menu_entry_args));
			    
  /* Create title for data view */
  label_args[0].value = (XtArgVal)XtNameToWidget(form,"Code_View");
  data_label = XtCreateManagedWidget("Data_Title",labelWidgetClass,form,
			    label_args,XtNumber(label_args));
			    
  XtOverrideTranslations(data_label,XtParseTranslationTable(label_trans));

  /* Create data view */
  view_args[0].value = (XtArgVal)data_label;
  w = XtCreateManagedWidget("Data_View",viewportWidgetClass,form,
			    view_args,XtNumber(view_args));

  /* Create initial list */
  {
    int i;
    
    data_arr = (char **)malloc((num_kb*1024+1)*sizeof(char *));

    for (i=0;i<num_kb*1024;i++)
      {
	data_arr[i] = (String)malloc(40);
	sprintf(data_arr[i],"%5x            %10x",i,mem[i]);
      }
    data_arr[num_kb*1024] = (String)malloc(40);
    *data_arr[num_kb*1024] = NULL;
  }

  list_args[0].value = (XtArgVal)data_arr;
  list_args[1].value = (XtArgVal)num_kb;
  list_args[3].value = (XtArgVal)(num_kb*1024);
  list_args[6].value = (XtArgVal)list_cbk;
  data_list = XtCreateManagedWidget("Data_List",memListWidgetClass,w,
			    list_args,XtNumber(list_args));
			    
  XtOverrideTranslations(data_list,XtParseTranslationTable(translations));
  
  return;
}

static void mem_octal(Widget w,XtPointer client_data,XtPointer call_data)
{
  mem_type = SHOW_MEM_OCTAL;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  /* Send message to simulator */
  Write_Comm_Channel_Str("Ro");
  
  return;
}

static void mem_decimal(Widget w,XtPointer client_data,XtPointer call_data)
{
  mem_type = SHOW_MEM_DECIMAL;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  /* Send message to simulator */
  Write_Comm_Channel_Str("Rd");

    return;
}

static void mem_hex(Widget w,XtPointer client_data,XtPointer call_data)
{
  mem_type = SHOW_MEM_HEX;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  /* Send message to simulator */
  Write_Comm_Channel_Str("Rx");

  return;
}

static void octal_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  val_type = SHOW_OCTAL;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  return;
}

static void sign_dec_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  val_type = SHOW_SIGNED_DEC;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  return;
}

static void unsign_dec_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  val_type = SHOW_UNSIGNED_DEC;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  return;
}

static void hex_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  val_type = SHOW_HEX;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  return;
}

static void ascii_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  val_type = SHOW_ASCII;

  if (data_type == SHOW_BYTES)
    bytes_val(w,client_data,call_data);
  else if (data_type == SHOW_HALFWORDS)
    half_word_val(w,client_data,call_data);
  else
    word_val(w,client_data,call_data);

  return;
}

static void bytes_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  int i=0;
  
  data_type = SHOW_BYTES;
  
  for (i=0;i<num_kb*1024;i++)
    {
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[i],"%5x            %12x",i,mem[i]);
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[i],"%5x            %12d",i,mem[i]);
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[i],"%5x            %12u",i,mem[i]);
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[i],"%5x            %12o",i,mem[i]);
	  else
	    sprintf(data_arr[i],"%5x            %12c",i,mem[i]);
	  break;
	case SHOW_MEM_DECIMAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[i],"%5d            %12x",i,mem[i]);
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[i],"%5d            %12d",i,mem[i]);
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[i],"%5d            %12u",i,mem[i]);
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[i],"%5d            %12o",i,mem[i]);
	  else
	    sprintf(data_arr[i],"%5d            %12c",i,mem[i]);
	  break;
	case SHOW_MEM_OCTAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[i],"%5o            %12x",i,mem[i]);
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[i],"%5o            %12d",i,mem[i]);
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[i],"%5o            %12u",i,mem[i]);
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[i],"%5o            %12o",i,mem[i]);
	  else
	    sprintf(data_arr[i],"%5o            %12c",i,mem[i]);
	  break;
	}
    }

  *data_arr[num_kb*1024] = NULL;
  Change_Data_Label();
  XawMemListChange(data_list,data_arr,num_kb*1024,0,True);
  if (current_data_selection > -1)
    {
      current_data_selection = -1;
    }
  return;
}

static void half_word_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  int i=0,j=0;
  char *ptr;
    
  data_type = SHOW_HALFWORDS;

  for (i=0,j=0;i<num_kb*1024;i+=2,j++)
    {
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5x            %12x",i,smem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5x            %12d",i,smem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5x            %12u",i,smem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5x            %12o",i,smem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5x                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	    }

	  break;
	case SHOW_MEM_DECIMAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5d            %12x",i,smem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5d            %12d",i,smem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5d            %12u",i,smem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5d            %12o",i,smem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5d                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	    }
	  break;
	case SHOW_MEM_OCTAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5o            %12x",i,smem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5o            %12d",i,smem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5o            %12u",i,smem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5o            %12o",i,smem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5o                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	    }
	    
	  break;
	}
    }
  *data_arr[num_kb*1024/2] = NULL;
  Change_Data_Label();
  XawMemListChange(data_list,data_arr,num_kb*1024/2,0,True);
  if (current_data_selection > -1)
    {
      current_data_selection = -1;
    }
  return;
}

static void word_val(Widget w,XtPointer client_data,XtPointer call_data)
{
  int i=0,j=0;
  char *ptr;
    
  data_type = SHOW_WORDS;

  for (i=0,j=0;i<num_kb*1024;i+=4,j++)
    {
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5x            %12x",i,imem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5x            %12d",i,imem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5x            %12u",i,imem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5x            %12o",i,imem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5x                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	      if (mem[i+2] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+2];
		  ptr[1] = '\0';
		}
	      if (mem[i+3] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+3];
		  ptr[1] = '\0';
		}
	    }
	  break;
	case SHOW_MEM_DECIMAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5d            %12x",i,imem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5d            %12d",i,imem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5d            %12u",i,imem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5d            %12o",i,imem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5d                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	      if (mem[i+2] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+2];
		  ptr[1] = '\0';
		}
	      if (mem[i+3] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+3];
		  ptr[1] = '\0';
		}
	    }
	  break;
	case SHOW_MEM_OCTAL:
	  if (val_type == SHOW_HEX)
	    sprintf(data_arr[j],"%5o            %12x",i,imem(i));
	  else if (val_type == SHOW_SIGNED_DEC)
	    sprintf(data_arr[j],"%5o            %12d",i,imem(i));
	  else if (val_type == SHOW_UNSIGNED_DEC)
	    sprintf(data_arr[j],"%5o            %12u",i,imem(i));
	  else if (val_type == SHOW_OCTAL)
	    sprintf(data_arr[j],"%5o            %12o",i,imem(i));
	  else
	    {
	      sprintf(data_arr[j],"%5o                      ",i);
	      if (mem[i] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i];
		  ptr[1] = '\0';
		}
	      if (mem[i+1] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+1];
		  ptr[1] = '\0';
		}
	      if (mem[i+2] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+2];
		  ptr[1] = '\0';
		}
	      if (mem[i+3] < 32)
		strcat(data_arr[j]," ");
	      else
		{
		  ptr = &data_arr[j][strlen(data_arr[j])];
		  ptr[0] = mem[i+3];
		  ptr[1] = '\0';
		}
	    }
	  break;
	}
    }
  *data_arr[num_kb*1024/4] = NULL;
  Change_Data_Label();
  XawMemListChange(data_list,data_arr,num_kb*1024/4,0,True);
  if (current_data_selection > -1)
    {
      current_data_selection = -1;
    }
  return;
}

static void edit_mem_location(Widget w,XtPointer client_data,XtPointer call_data)
{
  Widget widg = XtNameToWidget(form,"data_edit");
  Window junk1,junk2;
  int root_x,root_y,junkX,junkY;
  unsigned int junkM;
  XawMemListReturnStruct *listval = XawMemListShowCurrent(data_list);
  int index = listval->list_index;
  char buf[20];
  char *ptr;
    
  switch (val_type)
    {
    case SHOW_HEX:
      if (data_type == SHOW_BYTES)
	sprintf(buf,"%x",mem[index]);
      else if (data_type == SHOW_HALFWORDS)
	sprintf(buf,"%x",smem(index*2));
      else if (data_type == SHOW_WORDS)
	sprintf(buf,"%x",imem(index*4));
      break;
      
    case SHOW_SIGNED_DEC:
      if (data_type == SHOW_BYTES)
	sprintf(buf,"%d",mem[index]);
      else if (data_type == SHOW_HALFWORDS)
	sprintf(buf,"%d",smem(index*2));
      else if (data_type == SHOW_WORDS)
	sprintf(buf,"%d",imem(index*4));
      break;
      
    case SHOW_UNSIGNED_DEC:
      if (data_type == SHOW_BYTES)
	sprintf(buf,"%u",mem[index]);
      else if (data_type == SHOW_HALFWORDS)
	sprintf(buf,"%u",smem(index*2));
      else if (data_type == SHOW_WORDS)
	sprintf(buf,"%u",imem(index*4));
      break;
      
    case SHOW_OCTAL:
      if (data_type == SHOW_BYTES)
	sprintf(buf,"%o",mem[index]);
      else if (data_type == SHOW_HALFWORDS)
	sprintf(buf,"%o",smem(index*2));
      else if (data_type == SHOW_WORDS)
	sprintf(buf,"%o",imem(index*4));
      break;
      
    case SHOW_ASCII:
      buf[0] = '\0';
      if (data_type == SHOW_BYTES)
	{
	  if (mem[index] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index];
	      ptr[1] = '\0';
	    }
	}
      else if (data_type == SHOW_HALFWORDS)
	{
	  if (mem[index*2] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*2];
	      ptr[1] = '\0';
	    }
	  if (mem[index*2+1] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*2+1];
	      ptr[1] = '\0';
	    }
	}
      else if (data_type == SHOW_WORDS)
	{
	  if (mem[index*4] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*4];
	      ptr[1] = '\0';
	    }
	  if (mem[index*4+1] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*4+1];
	      ptr[1] = '\0';
	    }
	  if (mem[index*4+2] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*4+2];
	      ptr[1] = '\0';
	    }
	  if (mem[index*4+3] < 32)
	    strcat(buf," ");
	  else
	    {
	      ptr = &buf[strlen(buf)];
	      ptr[0] = mem[index*4+3];
	      ptr[1] = '\0';
	    }
	}
      break;
      
    }
  
  {
    static Arg args[] = {
      {XtNvalue, (XtArgVal)NULL},
    };
    args[0].value = (XtArgVal)buf;
    XtSetValues(dialog_widg,args,XtNumber(args));
  }
  
  XtRealizeWidget(widg);
  XQueryPointer(XtDisplay(widg),XtWindow(widg),&junk1,&junk2,
		&root_x,&root_y,&junkX,&junkY,&junkM);
  XtMoveWidget(widg,root_x-40,root_y-35);
  XtPopup(widg,XtGrabExclusive);
  
  return;
}

static void dialog_ok(Widget w,XtPointer client_data,XtPointer call_data)
{
  char *str;
  XawMemListReturnStruct *listval = XawMemListShowCurrent(data_list);
  int index = listval->list_index;
  char *ptr,mem_ch1,mem_ch2,mem_ch3,mem_ch4;
  unsigned short int mem_sint;
  unsigned int mem_uint;
  
  str = XawDialogGetValueString(dialog_widg);
  
  switch (data_type)
    {
    case SHOW_BYTES:
      switch (val_type)
	{
	case SHOW_HEX:
	  sscanf(str,"%x",&mem_uint);
	  mem[index] = mem_uint;
	  break;
	case SHOW_SIGNED_DEC:
	  sscanf(str,"%d",&mem_uint);
	  mem[index] = mem_uint;
	  break;
	case SHOW_UNSIGNED_DEC:
	  sscanf(str,"%u",&mem_uint);
	  mem[index] = mem_uint;
	  break;
	case SHOW_OCTAL:
	  sscanf(str,"%o",&mem_uint);
	  mem[index] = mem_uint;
	  break;
	case SHOW_ASCII:
	  sscanf(str,"%c",&mem_ch1);
	  mem[index] = mem_ch1;
	  break;
	}	  
      bytes_val(w,client_data,call_data);
      break;
    case SHOW_HALFWORDS:
      switch (val_type)
	{
	case SHOW_HEX:
	  sscanf(str,"%x",&mem_uint);
	  smem(index*2) = mem_uint;
	  break;
	case SHOW_SIGNED_DEC:
	  sscanf(str,"%d",&mem_uint);
	  smem(index*2) = mem_uint;
	  break;
	case SHOW_UNSIGNED_DEC:
	  sscanf(str,"%u",&mem_uint);
	  smem(index*2) = mem_uint;
	  break;
	case SHOW_OCTAL:
	  sscanf(str,"%o",&mem_uint);
	  smem(index*2) = mem_uint;
	  break;
	case SHOW_ASCII:
	  sscanf(str,"%c%c",&mem_ch1,&mem_ch2);
	  mem[index*2] = mem_ch1;
	  mem[index*2+1] = mem_ch2;
	  break;
	}	  
      half_word_val(w,client_data,call_data);
      break;
    case SHOW_WORDS:
      switch (val_type)
	{
	case SHOW_HEX:
	  sscanf(str,"%x",&mem_uint);
	  imem(index*4) = mem_uint;
	  break;
	case SHOW_SIGNED_DEC:
	  sscanf(str,"%d",&mem_uint);
	  imem(index*4) = mem_uint;
	  break;
	case SHOW_UNSIGNED_DEC:
	  sscanf(str,"%u",&mem_uint);
	  imem(index*4) = mem_uint;
	  break;
	case SHOW_OCTAL:
	  sscanf(str,"%o",&mem_uint);
	  imem(index*4) = mem_uint;
	  break;
	case SHOW_ASCII:
	  sscanf(str,"%c%c%c%c",&mem_ch1,&mem_ch2,&mem_ch3,&mem_ch4);
	  mem[index*4] = mem_ch1;
	  mem[index*4+1] = mem_ch2;
	  mem[index*4+2] = mem_ch3;
	  mem[index*4+3] = mem_ch4;
	  break;
	}	  
      word_val(w,client_data,call_data);
      break;
    }
    
  Update_Code();
  XtPopdown(XtParent(XtParent(w)));
  
  return;
}

static void dialog_cancel(Widget w,XtPointer client_data,XtPointer call_data)
{
  XtPopdown(XtParent(XtParent(w)));
  
  return;
}

static void data_select(Widget w,XtPointer client_data,XtPointer call_data)
{
  XawMemListReturnStruct *list_ret = XawMemListShowCurrent(data_list);
  
  if (list_ret->list_index == XAW_LIST_NONE)
    {
      selection_type = NO_SELECTION;
      mem_loc_select = -1;
      return;
    }
    
  if (selection_type == NO_SELECTION)
    {
      selection_type = DATA_SELECTED;
      mem_loc_select = list_ret->list_index;
      current_data_selection = mem_loc_select;
    }
  else if (selection_type == CODE_SELECTED)
    {
      selection_type = DATA_SELECTED;
      mem_loc_select = list_ret->list_index;
      return;
    }
  else
    {
      mem_loc_select = list_ret->list_index;
      return;
    }
    
  return;
}

void Change_Data_Label()
{
  char str[200];

  static Arg args[] = {
    {XtNlabel, (XtArgVal)NULL},
  };
  
  switch (val_type)
    {
    case SHOW_OCTAL:
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  sprintf(str,"DATA (address: HEX, data: OCT)");
	  break;
	case SHOW_MEM_DECIMAL:
	  sprintf(str,"DATA (address: UNS, data: OCT)");
	  break;
	case SHOW_MEM_OCTAL:
	  sprintf(str,"DATA (address: OCT, data: OCT)");
	  break;
	}
      break;
    case SHOW_SIGNED_DEC:
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  sprintf(str,"DATA (address: HEX, data: DEC)");
	  break;
	case SHOW_MEM_DECIMAL:
	  sprintf(str,"DATA (address: UNS, data: DEC)");
	  break;
	case SHOW_MEM_OCTAL:
	  sprintf(str,"DATA (address: OCT, data: DEC)");
	  break;
	}
      break;
    case SHOW_HEX:
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  sprintf(str,"DATA (address: HEX, data: HEX)");
	  break;
	case SHOW_MEM_DECIMAL:
	  sprintf(str,"DATA (address: UNS, data: HEX)");
	  break;
	case SHOW_MEM_OCTAL:
	  sprintf(str,"DATA (address: OCT, data: HEX)");
	  break;
	}
      break;
    case SHOW_ASCII:
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  sprintf(str,"DATA (address: HEX, data: ASC)");
	  break;
	case SHOW_MEM_DECIMAL:
	  sprintf(str,"DATA (address: UNS, data: ASC)");
	  break;
	case SHOW_MEM_OCTAL:
	  sprintf(str,"DATA (address: OCT, data: ASC)");
	  break;
	}
      break;
    case SHOW_UNSIGNED_DEC:
      switch (mem_type)
	{
	case SHOW_MEM_HEX:
	  sprintf(str,"DATA (address: HEX, data: UNS)");
	  break;
	case SHOW_MEM_DECIMAL:
	  sprintf(str,"DATA (address: UNS, data: UNS)");
	  break;
	case SHOW_MEM_OCTAL:
	  sprintf(str,"DATA (address: OCT, data: UNS)");
	  break;
	}
      break;
    }
    
  args[0].value = (XtArgVal)str;
  XtSetValues(data_label,args,XtNumber(args));

  return;
}

Update_Data()
{
  int i,j;
  char *ptr;
  
  switch (data_type)
    {
    case SHOW_WORDS:
      for (i=0,j=0;i<num_kb*1024;i+=4,j++)
	{
	  switch (mem_type)
	    {
	    case SHOW_MEM_HEX:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5x            %12x",i,imem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5x            %12d",i,imem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5x            %12u",i,imem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5x            %12o",i,imem(i));
	      else
		{
		  sprintf(data_arr[j],"%5x                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		  if (mem[i+2] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+2];
		      ptr[1] = '\0';
		    }
		  if (mem[i+3] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+3];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    case SHOW_MEM_DECIMAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5d            %12x",i,imem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5d            %12d",i,imem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5d            %12u",i,imem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5d            %12o",i,imem(i));
	      else
		{
		  sprintf(data_arr[j],"%5d                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		  if (mem[i+2] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+2];
		      ptr[1] = '\0';
		    }
		  if (mem[i+3] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+3];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    case SHOW_MEM_OCTAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5o            %12x",i,imem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5o            %12d",i,imem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5o            %12u",i,imem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5o            %12o",i,imem(i));
	      else
		{
		  sprintf(data_arr[j],"%5o                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		  if (mem[i+2] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+2];
		      ptr[1] = '\0';
		    }
		  if (mem[i+3] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+3];
		      ptr[1] = '\0';
		    }
		}
	      break;
	    }
	}
      *data_arr[num_kb*1024/4] = NULL;
      XawMemListChange(data_list,data_arr,num_kb*1024/4,0,True);
      break;

    case SHOW_BYTES:
      for (i=0;i<num_kb*1024;i++)
	{
	  switch (mem_type)
	    {
	    case SHOW_MEM_HEX:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[i],"%5x            %12x",i,mem[i]);
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[i],"%5x            %12d",i,mem[i]);
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[i],"%5x            %12u",i,mem[i]);
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[i],"%5x            %12o",i,mem[i]);
	      else
		{
		  sprintf(data_arr[j],"%5x                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		}
	      break;
	    case SHOW_MEM_DECIMAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[i],"%5d            %12x",i,mem[i]);
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[i],"%5d            %12d",i,mem[i]);
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[i],"%5d            %12u",i,mem[i]);
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[i],"%5d            %12o",i,mem[i]);
	      else
		{
		  sprintf(data_arr[j],"%5d                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		}
	      break;
	    case SHOW_MEM_OCTAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[i],"%5o            %12x",i,mem[i]);
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[i],"%5o            %12d",i,mem[i]);
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[i],"%5o            %12u",i,mem[i]);
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[i],"%5o            %12o",i,mem[i]);
	      else
		{
		  sprintf(data_arr[j],"%5o                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    }
	}
      *data_arr[num_kb*1024] = NULL;
      XawMemListChange(data_list,data_arr,num_kb*1024,0,True);
      break;
      
    case SHOW_HALFWORDS:
      for (i=0,j=0;i<num_kb*1024;i+=2,j++)
	{
	  switch (mem_type)
	    {
	    case SHOW_MEM_HEX:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5x            %12x",i,smem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5x            %12d",i,smem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5x            %12u",i,smem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5x            %12o",i,smem(i));
	      else
		{
		  sprintf(data_arr[j],"%5x                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    case SHOW_MEM_DECIMAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5d            %12x",i,smem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5d            %12d",i,smem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5d            %12u",i,smem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5d            %12o",i,smem(i));
	      else
		{
		  sprintf(data_arr[j],"%5d                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    case SHOW_MEM_OCTAL:
	      if (val_type == SHOW_HEX)
		sprintf(data_arr[j],"%5o            %12x",i,smem(i));
	      else if (val_type == SHOW_SIGNED_DEC)
		sprintf(data_arr[j],"%5o            %12d",i,smem(i));
	      else if (val_type == SHOW_UNSIGNED_DEC)
		sprintf(data_arr[j],"%5o            %12u",i,smem(i));
	      else if (val_type == SHOW_OCTAL)
		sprintf(data_arr[j],"%5o            %12o",i,smem(i));
	      else
		{
		  sprintf(data_arr[j],"%5o                      ",i);
		  if (mem[i] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i];
		      ptr[1] = '\0';
		    }
		  if (mem[i+1] < 32)
		    strcat(data_arr[j]," ");
		  else
		    {
		      ptr = &data_arr[j][strlen(data_arr[j])];
		      ptr[0] = mem[i+1];
		      ptr[1] = '\0';
		    }
		}
      
	      break;
	    }
	}
      *data_arr[num_kb*1024/2] = NULL;
      XawMemListChange(data_list,data_arr,num_kb*1024/2,0,True);
      break;
      
    }

  if (current_data_selection > -1)
    XawMemListHighlight(data_list,current_data_selection);
  
  return;
}



